import re
from .logger import Logger
from .providers import StoreProvider, Providers
from .stages import Stages
from .constants import *
from .exceptions import *


REGEX_PATTERN = r"^[a-zA-Z]([.a-zA-Z0-9_-]*[a-zA-ZA-Z0-9])?$"

class SecretProvider(StoreProvider):
    def list(self, config, uninherited=False, decrypt=False, filter=None):
        raise NotImplementedError()
    def add(self, config, key, value):
        raise NotImplementedError()
    def get(self, config, key, decrypt=False, revision=None):
        raise NotImplementedError()
    def history(self, config, key, decrypt=False):
        raise NotImplementedError()
    def delete(self, config, key):
        raise NotImplementedError()


class SecretService():

    def validate_key(self, key):
        Logger.info(f"Validating secret key '{key}'...")
        if not key:
            raise DSOException(MESSAGES['KeyNull'])
        if key == 'dso' or key.startswith('dso.'):
            raise DSOException(MESSAGES['DSOReserverdKey'].format(key))
        if not re.match(REGEX_PATTERN, key):
            raise DSOException(MESSAGES['InvalidKeyPattern'].format(key, REGEX_PATTERN))
        if '..' in key:
            raise DSOException(MESSAGES['InvalidKeyStr'].format(key, '..'))

    def list(self, config, uninherited=False, decrypt=False, filter=None):
        self.config = config
        project = config.project
        application = config.application
        stage = Stages.normalize(config.stage)
        provider = Providers.SecretProvider()
        Logger.info(f"Start listing secrets: project={project}, application={application}, stage={Stages.shorten(stage)}")
        return provider.list(config, uninherited, decrypt, filter)

    def add(self, config, key, value):
        self.config = config
        project = config.project
        application = config.application
        stage = Stages.normalize(config.stage)
        self.validate_key(key)
        provider = Providers.SecretProvider()
        Logger.info(f"Start adding secret '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
        return provider.add(config, key, value)

    def get(self, config, key, decrypt=False, revision=None):
        # self.validate_key(key)
        self.config = config
        project = config.project
        application = config.application
        stage = Stages.normalize(config.stage)
        provider = Providers.SecretProvider()
        Logger.info(f"Start getting the details of secret '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
        return provider.get(config, key, decrypt, revision)

    def history(self, config, key, decrypt=False):
        # self.validate_key(key)
        self.config = config
        project = config.project
        application = config.application
        stage = Stages.normalize(config.stage)
        provider = Providers.SecretProvider()
        Logger.info(f"Start getting the history of secret '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
        return provider.history(config, key, decrypt)

    def delete(self, config, key):
        # self.validate_key(key)
        self.config = config
        project = config.project
        application = config.application
        stage = Stages.normalize(config.stage)
        provider = Providers.SecretProvider()
        Logger.info(f"Start deleting secret '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
        return provider.delete(config, key)

Secrets = SecretService()
